<?php
// $Id: hooks.php,v 1.6 2010/01/12 10:29:56 nickurbits Exp $
/**
 * @file
 *  Contains API documentation for licensing module
 */
 
/**
 * Implement hook_licensingapi
 *
 * Perform an action during certain stages of a license's lifetime
 *
 * @param string $op
 *   Operation being performed. Possible values are:
 *     - load: When a license object is being loaded and built from the database
 *     - presave: The license object is being prepared for saving
 *     - postsave: The license object has just been saved and values updated
 *     - generate: Any changes to the license key should be made with 
 *       $license->set_key()
 *     - userdel: User is being deleted - modules should remove all data 
 *       associated with the account during this operation.
 * @param object (license) $license
 *   Current license object
 * @param object stdClass $account
 *   User account when performing user actions
 * @param string $brand
 *   During 'generate' operation, contains license branding string
 * @param string $keyword
 *   During 'generate' operation, contains license secret keyword for use as salt
 *   in license generation algorithm
 * @see licensing_licensingapi()
 */
function hook_licensingapi($op, &$license, &$account = NULL, $brand = NULL, $keyword = NULL) {
  if ($op == 'load') {
    // We are going to load our extra data onto the license object
    $result = db_query("SELECT * FROM {custom_licensing_module} clm WHERE clm.lid = %d LIMIT 1;", $license->get_lid());
    $additions = db_fetch_array($result);
    $license->custom_licensing_module = $additions;
  }
  if ($op == 'generate') {
    // we're going to prepend 3 characters to the start of the license key
    $hash = md5($keyword . time());
    $chars = substr($hash, 5, 8);
    $license->set_key($chars .'-'. $license->get_key()); 
  }
  if ($op == 'userdel') {
    // we're deleting all our custom data relating to this account
    db_query("DELETE FROM {custom_licensing_module} clm WHERE clm.uid = %d", $account->uid);
  }
}

/**
 * Implement hook_licensing_register
 *
 * Perform actions during various stages of the license registration process
 *
 * @param string $op
 *   Operation being performed, possible values are:
 *     - form: registration form is being presented, insert your own form fields
 *     - validate: registration form has been submitted, do your validation here
 *     - submit: registration form has passed validation, do what you will with
 *       your data
 * @param object (license) $license
 *   Current license object if available
 * @param array $form_values
 *   Contains submitted values from $form_state['values'] during following ops:
 *     - validate
 *     - submit
 * @param string $form_id
 *   Contains the form id if available
 * @return 
 *   Varies depending on the operation
 *     - form: array of form fields to be presented alongside the license field
 *     - everything else: none
 */
function hook_licensing_register($op, $license = NULL, $form_values = NULL, $form_id = NULL) {
  if ($op == 'form') {
    // present a software version select list next to the license field
    $form = array(
      'version' => array(
        '#title' => t('Software version'),
        '#description' =. t('Select the version of software you purchased and would like to register'),
        '#type' => 'select',
        '#options' => array(
          '2010' => t('Software v2010'),
          '2009' => t('Software v2009'),
        ),
      ),
    );
    if ($form_id == 'user_register') {
      // This makes sure our field is only mandatory if the license field is on the user_register form
      $form['version']['#required'] = variable_get('licensing_user_register_mandatory', LICENSING_USER_REGISTER_MANDATORY_DEFAULT);
    }
    else {
      // otherwise, it is probably required
      $form['version']['#required'] = TRUE;
    }
    
    // return our form array
    return $form;
  }
  if ($op == 'validate') {
    // make sure our license key matches up to the version they selected
    $sql = "SELECT version FROM {custom_licensing_module_versions} WHERE lid = %d LIMIT 1";
    $version = db_result(db_query($sql, $license->get_lid()));
    if ($version != $form_values['version']) {
      // version from our table doesnt match the selected version
      form_set_error('version', t('This license key is not for that version of software, please select the version you purchased.'));
    }
  }
}

/**
 * Implement hook_licensing_operations
 *
 * Present various links on a per-license basis on the licensing administration view
 *
 * @param license $license
 *   Current license object
 * @param string $path
 *   Current drupal internal path
 * @return array
 *   associative array of 'operation link title' => 'link path'
 */
function hook_licensing_operations($license, $path) {
  $ops = array();
  if ($license->get_uid()) {
    // make sure we only present this link if a user has registered this license
    $ops['delete user'] = 'user/'. $license->get_uid() .'/delete';
  }
  
  return $ops;
}

/**
 * If you wish to override what class is loaded in place of the default 'license' class, read this.
 *
 * As you can see at the top of licensing.module
 *
 * // Allow other modules to specify which class to load, they should be child
 * // classes of license though.
 * define('LICENSING_CLASS', variable_get('licensing_class', 'license'));
 *
 * Simply by doing a variable_set in your module's hook_enable (and removing it in hook_disable), 
 * you can use your own child class in place of the license class. See this example:
 */
function hook_enable() {
  variable_set('licensing_class', 'custom_class');
}

// Be sure to delete the variable when the module is disabled
function hook_disable() {
  variable_del('licensing_class');
}